@@ -3,9 +3,9 @@
 use std::path::{Path, PathBuf};
 use std::sync::Mutex;
 
-use git2;
+use log::*;
 
-use fs::fields as f;
+use crate::fs::fields as f;
 
 
 /// A **Git cache** is assembled based on the user’s input arguments.
@@ -36,9 +36,11 @@ impl GitCache {
 
 use std::iter::FromIterator;
 impl FromIterator<PathBuf> for GitCache {
-    fn from_iter<I: IntoIterator<Item=PathBuf>>(iter: I) -> Self {
+    fn from_iter<I>(iter: I) -> Self
+    where I: IntoIterator<Item=PathBuf>
+    {
         let iter = iter.into_iter();
-        let mut git = GitCache {
+        let mut git = Self {
             repos: Vec::with_capacity(iter.size_hint().0),
             misses: Vec::new(),
         };
@@ -61,8 +63,10 @@ impl FromIterator<PathBuf> for GitCache {
 
                         debug!("Discovered new Git repo");
                         git.repos.push(r);
-                    },
-                    Err(miss) => git.misses.push(miss),
+                    }
+                    Err(miss) => {
+                        git.misses.push(miss)
+                    }
                 }
             }
         }
@@ -72,8 +76,6 @@ impl FromIterator<PathBuf> for GitCache {
 }
 
 
-
-
 /// A **Git repository** is one we’ve discovered somewhere on the filesystem.
 pub struct GitRepo {
 
@@ -99,7 +101,9 @@ pub struct GitRepo {
 enum GitContents {
 
     /// All the interesting Git stuff goes through this.
-    Before { repo: git2::Repository },
+    Before {
+        repo: git2::Repository,
+    },
 
     /// Temporary value used in `repo_to_statuses` so we can move the
     /// repository out of the `Before` variant.
@@ -107,7 +111,9 @@ enum GitContents {
 
     /// The data we’ve extracted from the repository, but only after we’ve
     /// actually done so.
-    After { statuses: Git }
+    After {
+        statuses: Git,
+    },
 }
 
 impl GitRepo {
@@ -116,27 +122,26 @@ impl GitRepo {
     /// depending on the prefix-lookup flag) and returns its Git status.
     ///
     /// Actually querying the `git2` repository for the mapping of paths to
-    /// Git statuses is only done once, and gets cached so we don't need to
+    /// Git statuses is only done once, and gets cached so we don’t need to
     /// re-query the entire repository the times after that.
     ///
     /// The temporary `Processing` enum variant is used after the `git2`
     /// repository is moved out, but before the results have been moved in!
-    /// See https://stackoverflow.com/q/45985827/3484614
+    /// See <https://stackoverflow.com/q/45985827/3484614>
     fn search(&self, index: &Path, prefix_lookup: bool) -> f::Git {
-        use self::GitContents::*;
         use std::mem::replace;
 
         let mut contents = self.contents.lock().unwrap();
-        if let After { ref statuses } = *contents {
+        if let GitContents::After { ref statuses } = *contents {
             debug!("Git repo {:?} has been found in cache", &self.workdir);
             return statuses.status(index, prefix_lookup);
         }
 
         debug!("Querying Git repo {:?} for the first time", &self.workdir);
-        let repo = replace(&mut *contents, Processing).inner_repo();
+        let repo = replace(&mut *contents, GitContents::Processing).inner_repo();
         let statuses = repo_to_statuses(&repo, &self.workdir);
         let result = statuses.status(index, prefix_lookup);
-        let _processing = replace(&mut *contents, After { statuses });
+        let _processing = replace(&mut *contents, GitContents::After { statuses });
         result
     }
 
@@ -152,7 +157,7 @@ impl GitRepo {
 
     /// Searches for a Git repository at any point above the given path.
     /// Returns the original buffer if none is found.
-    fn discover(path: PathBuf) -> Result<GitRepo, PathBuf> {
+    fn discover(path: PathBuf) -> Result<Self, PathBuf> {
         info!("Searching for Git repository above {:?}", path);
         let repo = match git2::Repository::discover(&path) {
             Ok(r) => r,
@@ -162,15 +167,14 @@ impl GitRepo {
             }
         };
 
-        match repo.workdir().map(|wd| wd.to_path_buf()) {
-            Some(workdir) => {
-                let contents = Mutex::new(GitContents::Before { repo });
-                Ok(GitRepo { contents, workdir, original_path: path, extra_paths: Vec::new() })
-            },
-            None => {
-                warn!("Repository has no workdir?");
-                Err(path)
-            }
+        if let Some(workdir) = repo.workdir() {
+            let workdir = workdir.to_path_buf();
+            let contents = Mutex::new(GitContents::Before { repo });
+            Ok(Self { contents, workdir, original_path: path, extra_paths: Vec::new() })
+        }
+        else {
+            warn!("Repository has no workdir?");
+            Err(path)
         }
     }
 }
@@ -181,7 +185,7 @@ impl GitContents {
     /// (consuming the value) if it has. This is needed because the entire
     /// enum variant gets replaced when a repo is queried (see above).
     fn inner_repo(self) -> git2::Repository {
-        if let GitContents::Before { repo } = self {
+        if let Self::Before { repo } = self {
             repo
         }
         else {
@@ -205,8 +209,10 @@ fn repo_to_statuses(repo: &git2::Repository, workdir: &Path) -> Git {
                 let elem = (path, e.status());
                 statuses.push(elem);
             }
-        },
-        Err(e) => error!("Error looking up Git statuses: {:?}", e),
+        }
+        Err(e) => {
+            error!("Error looking up Git statuses: {:?}", e);
+        }
     }
 
     Git { statuses }
@@ -217,7 +223,7 @@ fn repo_to_statuses(repo: &git2::Repository, workdir: &Path) -> Git {
 //   20.311276  INFO:exa::fs::feature::git: Getting Git statuses for repo with workdir "/vagrant/"
 //   20.799610  DEBUG:exa::output::table: Getting Git status for file "./Cargo.toml"
 //
-// Even inserting another logging line immediately afterwards doesn't make it
+// Even inserting another logging line immediately afterwards doesn’t make it
 // look any faster.
 
 
@@ -236,40 +242,62 @@ impl Git {
                     else { self.file_status(index) }
     }
 
-    /// Get the status for the file at the given path.
+    /// Get the user-facing status of a file.
+    /// We check the statuses directly applying to a file, and for the ignored
+    /// status we check if any of its parents directories is ignored by git.
     fn file_status(&self, file: &Path) -> f::Git {
         let path = reorient(file);
-        self.statuses.iter()
-            .find(|p| p.0.as_path() == path)
-            .map(|&(_, s)| f::Git { staged: index_status(s), unstaged: working_tree_status(s) })
-            .unwrap_or_default()
+
+        let s = self.statuses.iter()
+            .filter(|p| if p.1 == git2::Status::IGNORED {
+                path.starts_with(&p.0)
+            } else {
+                p.0 == path
+            })
+            .fold(git2::Status::empty(), |a, b| a | b.1);
+
+        let staged = index_status(s);
+        let unstaged = working_tree_status(s);
+        f::Git { staged, unstaged }
     }
 
-    /// Get the combined status for all the files whose paths begin with the
-    /// path that gets passed in. This is used for getting the status of
-    /// directories, which don’t really have an ‘official’ status.
+    /// Get the combined, user-facing status of a directory.
+    /// Statuses are aggregating (for example, a directory is considered
+    /// modified if any file under it has the status modified), except for
+    /// ignored status which applies to files under (for example, a directory
+    /// is considered ignored if one of its parent directories is ignored).
     fn dir_status(&self, dir: &Path) -> f::Git {
         let path = reorient(dir);
-        let s = self.statuses.iter()
-                             .filter(|p| p.0.starts_with(&path))
-                             .fold(git2::Status::empty(), |a, b| a | b.1);
 
-        f::Git { staged: index_status(s), unstaged: working_tree_status(s) }
+        let s = self.statuses.iter()
+            .filter(|p| if p.1 == git2::Status::IGNORED {
+                path.starts_with(&p.0)
+            } else {
+                p.0.starts_with(&path)
+            })
+            .fold(git2::Status::empty(), |a, b| a | b.1);
+
+        let staged = index_status(s);
+        let unstaged = working_tree_status(s);
+        f::Git { staged, unstaged }
     }
 }
 
+
 /// Converts a path to an absolute path based on the current directory.
 /// Paths need to be absolute for them to be compared properly, otherwise
 /// you’d ask a repo about “./README.md” but it only knows about
-/// “/vagrant/REAMDE.md”, prefixed by the workdir.
+/// “/vagrant/README.md”, prefixed by the workdir.
 fn reorient(path: &Path) -> PathBuf {
     use std::env::current_dir;
-    // I’m not 100% on this func tbh
-    match current_dir() {
-        Err(_)  => Path::new(".").join(&path),
-        Ok(dir) => dir.join(&path),
-    }.canonicalize().unwrap()   // errors can be ignored here because they only occur if
-                                // the path does not exist / a component is not a folder
+
+    // TODO: I’m not 100% on this func tbh
+    let path = match current_dir() {
+        Err(_)   => Path::new(".").join(&path),
+        Ok(dir)  => dir.join(&path),
+    };
+
+    path.canonicalize().unwrap_or(path)
 }
 
 /// The character to display if the file has been modified, but not staged.
@@ -281,6 +309,7 @@ fn working_tree_status(status: git2::Status) -> f::GitStatus {
         s if s.contains(git2::Status::WT_RENAMED)     => f::GitStatus::Renamed,
         s if s.contains(git2::Status::WT_TYPECHANGE)  => f::GitStatus::TypeChange,
         s if s.contains(git2::Status::IGNORED)        => f::GitStatus::Ignored,
+        s if s.contains(git2::Status::CONFLICTED)     => f::GitStatus::Conflicted,
         _                                             => f::GitStatus::NotModified,
     }
 }
