@@ -60,7 +60,7 @@
 //!
 //! `--sort=size` should override `--sort=Name` because it’s closer to the end
 //! of the arguments array. In fact, because there’s no way to tell where the
-//! arguments came from -- it’s just a heuristic -- this will still work even
+//! arguments came from — it’s just a heuristic — this will still work even
 //! if no aliases are being used!
 //!
 //! Finally, this isn’t just useful when options could override each other.
@@ -69,32 +69,34 @@
 //! it’s clear what the user wants.
 
 
-use std::ffi::{OsStr, OsString};
+use std::ffi::OsStr;
 
-use fs::dir_action::DirAction;
-use fs::filter::FileFilter;
-use output::{View, Mode, details, grid_details};
+use crate::fs::dir_action::DirAction;
+use crate::fs::filter::{FileFilter, GitIgnore};
+use crate::output::{View, Mode, details, grid_details};
+use crate::theme::Options as ThemeOptions;
 
-mod style;
 mod dir_action;
+mod file_name;
 mod filter;
+mod flags;
+mod theme;
 mod view;
 
+mod error;
+pub use self::error::{OptionsError, NumberSource};
+
 mod help;
 use self::help::HelpString;
 
-mod version;
-use self::version::VersionString;
-
-mod misfire;
-pub use self::misfire::Misfire;
+mod parser;
+use self::parser::MatchedFlags;
 
 pub mod vars;
 pub use self::vars::Vars;
 
-mod parser;
-mod flags;
-use self::parser::MatchedFlags;
+mod version;
+use self::version::VersionString;
 
 
 /// These **options** represent a parsed, error-checked versions of the
@@ -109,8 +111,14 @@ pub struct Options {
     /// How to sort and filter files before outputting them.
     pub filter: FileFilter,
 
-    /// The type of output to use (lines, grid, or details).
+    /// The user’s preference of view to use (lines, grid, details, or
+    /// grid-details) along with the options on how to render file names.
+    /// If the view requires the terminal to have a width, and there is no
+    /// width, then the view will be downgraded.
     pub view: View,
+
+    /// The options to make up the styles of the UI and file names.
+    pub theme: ThemeOptions,
 }
 
 impl Options {
@@ -119,58 +127,94 @@ impl Options {
     /// struct and a list of free filenames, using the environment variables
     /// for extra options.
     #[allow(unused_results)]
-    pub fn parse<'args, I, V>(args: I, vars: &V) -> Result<(Options, Vec<&'args OsStr>), Misfire>
-    where I: IntoIterator<Item=&'args OsString>,
-          V: Vars {
-        use options::parser::{Matches, Strictness};
+    pub fn parse<'args, I, V>(args: I, vars: &V) -> OptionsResult<'args>
+    where I: IntoIterator<Item = &'args OsStr>,
+          V: Vars,
+    {
+        use crate::options::parser::{Matches, Strictness};
 
         let strictness = match vars.get(vars::EXA_STRICT) {
             None                         => Strictness::UseLastArguments,
             Some(ref t) if t.is_empty()  => Strictness::UseLastArguments,
-            _                            => Strictness::ComplainAboutRedundantArguments,
+            Some(_)                      => Strictness::ComplainAboutRedundantArguments,
         };
 
         let Matches { flags, frees } = match flags::ALL_ARGS.parse(args, strictness) {
-            Ok(m)   => m,
-            Err(e)  => return Err(Misfire::InvalidOptions(e)),
+            Ok(m)    => m,
+            Err(pe)  => return OptionsResult::InvalidOptions(OptionsError::Parse(pe)),
         };
 
-        HelpString::deduce(&flags).map_err(Misfire::Help)?;
-        VersionString::deduce(&flags).map_err(Misfire::Version)?;
+        if let Some(help) = HelpString::deduce(&flags) {
+            return OptionsResult::Help(help);
+        }
 
-        let options = Options::deduce(&flags, vars)?;
-        Ok((options, frees))
+        if let Some(version) = VersionString::deduce(&flags) {
+            return OptionsResult::Version(version);
+        }
+
+        match Self::deduce(&flags, vars) {
+            Ok(options)  => OptionsResult::Ok(options, frees),
+            Err(oe)      => OptionsResult::InvalidOptions(oe),
+        }
     }
 
     /// Whether the View specified in this set of options includes a Git
     /// status column. It’s only worth trying to discover a repository if the
     /// results will end up being displayed.
     pub fn should_scan_for_git(&self) -> bool {
+        if self.filter.git_ignore == GitIgnore::CheckAndIgnore {
+            return true;
+        }
+
         match self.view.mode {
             Mode::Details(details::Options { table: Some(ref table), .. }) |
-            Mode::GridDetails(grid_details::Options { details: details::Options { table: Some(ref table), .. }, .. }) => table.extra_columns.git,
+            Mode::GridDetails(grid_details::Options { details: details::Options { table: Some(ref table), .. }, .. }) => table.columns.git,
             _ => false,
         }
     }
 
     /// Determines the complete set of options based on the given command-line
     /// arguments, after they’ve been parsed.
-    fn deduce<V: Vars>(matches: &MatchedFlags, vars: &V) -> Result<Options, Misfire> {
-        let dir_action = DirAction::deduce(matches)?;
-        let filter = FileFilter::deduce(matches)?;
-        let view = View::deduce(matches, vars)?;
+    fn deduce<V: Vars>(matches: &MatchedFlags<'_>, vars: &V) -> Result<Self, OptionsError> {
+        if cfg!(not(feature = "git")) &&
+                matches.has_where_any(|f| f.matches(&flags::GIT) || f.matches(&flags::GIT_IGNORE)).is_some() {
+            return Err(OptionsError::Unsupported(format!(
+                "Options --git and --git-ignore can't be used because `git` feature was disabled in this build of exa"
+            )));
+        }
 
-        Ok(Options { dir_action, view, filter })
+        let view = View::deduce(matches, vars)?;
+        let dir_action = DirAction::deduce(matches, matches!(view.mode, Mode::Details(_)))?;
+        let filter = FileFilter::deduce(matches)?;
+        let theme = ThemeOptions::deduce(matches, vars)?;
+
+        Ok(Self { dir_action, filter, view, theme })
     }
 }
 
 
+/// The result of the `Options::getopts` function.
+#[derive(Debug)]
+pub enum OptionsResult<'args> {
+
+    /// The options were parsed successfully.
+    Ok(Options, Vec<&'args OsStr>),
+
+    /// There was an error parsing the arguments.
+    InvalidOptions(OptionsError),
+
+    /// One of the arguments was `--help`, so display help.
+    Help(HelpString),
+
+    /// One of the arguments was `--version`, so display the version number.
+    Version(VersionString),
+}
+
 
 #[cfg(test)]
 pub mod test {
-    use super::{Options, Misfire, flags};
-    use options::parser::{Arg, MatchedFlags};
-    use std::ffi::OsString;
+    use crate::options::parser::{Arg, MatchedFlags};
+    use std::ffi::OsStr;
 
     #[derive(PartialEq, Debug)]
     pub enum Strictnesses {
@@ -186,60 +230,24 @@ pub mod test {
     /// It returns a vector with one or two elements in.
     /// These elements can then be tested with assert_eq or what have you.
     pub fn parse_for_test<T, F>(inputs: &[&str], args: &'static [&'static Arg], strictnesses: Strictnesses, get: F) -> Vec<T>
-    where F: Fn(&MatchedFlags) -> T
+    where F: Fn(&MatchedFlags<'_>) -> T
     {
         use self::Strictnesses::*;
-        use options::parser::{Args, Strictness};
+        use crate::options::parser::{Args, Strictness};
 
-        let bits = inputs.into_iter().map(|&o| os(o)).collect::<Vec<OsString>>();
+        let bits = inputs.into_iter().map(OsStr::new).collect::<Vec<_>>();
         let mut result = Vec::new();
 
         if strictnesses == Last || strictnesses == Both {
-            let results = Args(args).parse(bits.iter(), Strictness::UseLastArguments);
+            let results = Args(args).parse(bits.clone(), Strictness::UseLastArguments);
             result.push(get(&results.unwrap().flags));
         }
 
         if strictnesses == Complain || strictnesses == Both {
-            let results = Args(args).parse(bits.iter(), Strictness::ComplainAboutRedundantArguments);
+            let results = Args(args).parse(bits, Strictness::ComplainAboutRedundantArguments);
             result.push(get(&results.unwrap().flags));
         }
 
         result
     }
-
-    /// Creates an `OSStr` (used in tests)
-    #[cfg(test)]
-    fn os(input: &str) -> OsString {
-        let mut os = OsString::new();
-        os.push(input);
-        os
-    }
-
-    #[test]
-    fn files() {
-        let args = [ os("this file"), os("that file") ];
-        let outs = Options::parse(&args, &None).unwrap().1;
-        assert_eq!(outs, vec![ &os("this file"), &os("that file") ])
-    }
-
-    #[test]
-    fn no_args() {
-        let nothing: Vec<OsString> = Vec::new();
-        let outs = Options::parse(&nothing, &None).unwrap().1;
-        assert!(outs.is_empty());  // Listing the `.` directory is done in main.rs
-    }
-
-    #[test]
-    fn long_across() {
-        let args = [ os("--long"), os("--across") ];
-        let opts = Options::parse(&args, &None);
-        assert_eq!(opts.unwrap_err(), Misfire::Useless(&flags::ACROSS, true, &flags::LONG))
-    }
-
-    #[test]
-    fn oneline_across() {
-        let args = [ os("--oneline"), os("--across") ];
-        let opts = Options::parse(&args, &None);
-        assert_eq!(opts.unwrap_err(), Misfire::Useless(&flags::ACROSS, true, &flags::ONE_LINE))
-    }
 }
