@@ -1,30 +1,38 @@
 use ansi_term::Style;
 use users::{Users, Groups};
 
-use fs::fields as f;
-use output::cell::TextCell;
+use crate::fs::fields as f;
+use crate::output::cell::TextCell;
+use crate::output::table::UserFormat;
 
 
 impl f::Group {
-    pub fn render<C: Colours, U: Users+Groups>(&self, colours: &C, users: &U) -> TextCell {
+    pub fn render<C: Colours, U: Users+Groups>(self, colours: &C, users: &U, format: UserFormat) -> TextCell {
         use users::os::unix::GroupExt;
 
         let mut style = colours.not_yours();
 
         let group = match users.get_group_by_gid(self.0) {
-            Some(g) => (*g).clone(),
-            None    => return TextCell::paint(style, self.0.to_string()),
+            Some(g)  => (*g).clone(),
+            None     => return TextCell::paint(style, self.0.to_string()),
         };
 
         let current_uid = users.get_current_uid();
         if let Some(current_user) = users.get_user_by_uid(current_uid) {
+
             if current_user.primary_group_id() == group.gid()
-            || group.members().iter().any(|u| u == current_user.name()) {
+            || group.members().iter().any(|u| u == current_user.name())
+            {
                 style = colours.yours();
             }
         }
 
-        TextCell::paint(style, group.name().to_string_lossy().into())
+        let group_name = match format {
+            UserFormat::Name => group.name().to_string_lossy().into(),
+            UserFormat::Numeric => group.gid().to_string(),
+        };
+
+        TextCell::paint(style, group_name)
     }
 }
 
@@ -39,8 +47,9 @@ pub trait Colours {
 #[allow(unused_results)]
 pub mod test {
     use super::Colours;
-    use fs::fields as f;
-    use output::cell::TextCell;
+    use crate::fs::fields as f;
+    use crate::output::cell::TextCell;
+    use crate::output::table::UserFormat;
 
     use users::{User, Group};
     use users::mock::MockUsers;
@@ -64,16 +73,21 @@ pub mod test {
 
         let group = f::Group(100);
         let expected = TextCell::paint_str(Fixed(81).normal(), "folk");
-        assert_eq!(expected, group.render(&TestColours, &users))
+        assert_eq!(expected, group.render(&TestColours, &users, UserFormat::Name));
+
+        let expected = TextCell::paint_str(Fixed(81).normal(), "100");
+        assert_eq!(expected, group.render(&TestColours, &users, UserFormat::Numeric));
     }
 
+
     #[test]
     fn unnamed() {
         let users = MockUsers::with_current_uid(1000);
 
         let group = f::Group(100);
         let expected = TextCell::paint_str(Fixed(81).normal(), "100");
-        assert_eq!(expected, group.render(&TestColours, &users));
+        assert_eq!(expected, group.render(&TestColours, &users, UserFormat::Name));
+        assert_eq!(expected, group.render(&TestColours, &users, UserFormat::Numeric));
     }
 
     #[test]
@@ -84,7 +98,7 @@ pub mod test {
 
         let group = f::Group(100);
         let expected = TextCell::paint_str(Fixed(80).normal(), "folk");
-        assert_eq!(expected, group.render(&TestColours, &users))
+        assert_eq!(expected, group.render(&TestColours, &users, UserFormat::Name))
     }
 
     #[test]
@@ -97,13 +111,13 @@ pub mod test {
 
         let group = f::Group(100);
         let expected = TextCell::paint_str(Fixed(80).normal(), "folk");
-        assert_eq!(expected, group.render(&TestColours, &users))
+        assert_eq!(expected, group.render(&TestColours, &users, UserFormat::Name))
     }
 
     #[test]
     fn overflow() {
         let group = f::Group(2_147_483_648);
         let expected = TextCell::paint_str(Fixed(81).normal(), "2147483648");
-        assert_eq!(expected, group.render(&TestColours, &MockUsers::with_current_uid(0)));
+        assert_eq!(expected, group.render(&TestColours, &MockUsers::with_current_uid(0), UserFormat::Numeric));
     }
 }
