@@ -1,49 +1,58 @@
 use ansi_term::Style;
 use locale::Numeric as NumericLocale;
+use number_prefix::Prefix;
 
-use fs::fields as f;
-use output::cell::{TextCell, DisplayWidth};
-use output::table::SizeFormat;
-
+use crate::fs::fields as f;
+use crate::output::cell::{TextCell, DisplayWidth};
+use crate::output::table::SizeFormat;
 
 
 impl f::Size {
-    pub fn render<C: Colours>(&self, colours: &C, size_format: SizeFormat, numerics: &NumericLocale) -> TextCell {
-        use number_prefix::{Prefixed, Standalone, NumberPrefix, PrefixNames};
+    pub fn render<C: Colours>(self, colours: &C, size_format: SizeFormat, numerics: &NumericLocale) -> TextCell {
+        use number_prefix::NumberPrefix;
 
-        let size = match *self {
-            f::Size::Some(s)             => s,
-            f::Size::None                => return TextCell::blank(colours.no_size()),
-            f::Size::DeviceIDs(ref ids)  => return ids.render(colours),
+        let size = match self {
+            Self::Some(s)             => s,
+            Self::None                => return TextCell::blank(colours.no_size()),
+            Self::DeviceIDs(ref ids)  => return ids.render(colours),
         };
 
         let result = match size_format {
             SizeFormat::DecimalBytes  => NumberPrefix::decimal(size as f64),
             SizeFormat::BinaryBytes   => NumberPrefix::binary(size as f64),
             SizeFormat::JustBytes     => {
+
+                // Use the binary prefix to select a style.
+                let prefix = match NumberPrefix::binary(size as f64) {
+                    NumberPrefix::Standalone(_)   => None,
+                    NumberPrefix::Prefixed(p, _)  => Some(p),
+                };
+
+                // But format the number directly using the locale.
                 let string = numerics.format_int(size);
-                return TextCell::paint(colours.size(size), string);
-            },
+
+                return TextCell::paint(colours.size(prefix), string);
+            }
         };
 
         let (prefix, n) = match result {
-            Standalone(b)  => return TextCell::paint(colours.size(b as u64), b.to_string()),
-            Prefixed(p, n) => (p, n)
+            NumberPrefix::Standalone(b)   => return TextCell::paint(colours.size(None), numerics.format_int(b)),
+            NumberPrefix::Prefixed(p, n)  => (p, n),
         };
 
         let symbol = prefix.symbol();
-        let number = if n < 10f64 { numerics.format_float(n, 1) }
-                             else { numerics.format_int(n as isize) };
-
-        // The numbers and symbols are guaranteed to be written in ASCII, so
-        // we can skip the display width calculation.
-        let width = DisplayWidth::from(number.len() + symbol.len());
-
+        let number = if n < 10_f64 {
+            numerics.format_float(n, 1)
+        } else {
+            numerics.format_int(n.round() as isize)
+        };
+        
         TextCell {
-            width,
+            // symbol is guaranteed to be ASCII since unit prefixes are hardcoded.
+            width: DisplayWidth::from(&*number) + symbol.len(),
             contents: vec![
-                colours.size(size).paint(number),
-                colours.unit().paint(symbol),
+                colours.size(Some(prefix)).paint(number),
+                colours.unit(Some(prefix)).paint(symbol),
             ].into(),
         }
     }
@@ -51,7 +60,7 @@ impl f::Size {
 
 
 impl f::DeviceIDs {
-    fn render<C: Colours>(&self, colours: &C) -> TextCell {
+    fn render<C: Colours>(self, colours: &C) -> TextCell {
         let major = self.major.to_string();
         let minor = self.minor.to_string();
 
@@ -68,8 +77,8 @@ impl f::DeviceIDs {
 
 
 pub trait Colours {
-    fn size(&self, size: u64) -> Style;
-    fn unit(&self) -> Style;
+    fn size(&self, prefix: Option<Prefix>) -> Style;
+    fn unit(&self, prefix: Option<Prefix>) -> Style;
     fn no_size(&self) -> Style;
 
     fn major(&self) -> Style;
@@ -81,20 +90,21 @@ pub trait Colours {
 #[cfg(test)]
 pub mod test {
     use super::Colours;
-    use output::cell::{TextCell, DisplayWidth};
-    use output::table::SizeFormat;
-    use fs::fields as f;
+    use crate::output::cell::{TextCell, DisplayWidth};
+    use crate::output::table::SizeFormat;
+    use crate::fs::fields as f;
 
     use locale::Numeric as NumericLocale;
     use ansi_term::Colour::*;
     use ansi_term::Style;
+    use number_prefix::Prefix;
 
 
     struct TestColours;
 
     impl Colours for TestColours {
-        fn size(&self, _size: u64) -> Style { Fixed(66).normal() }
-        fn unit(&self)             -> Style { Fixed(77).bold() }
+        fn size(&self, _prefix: Option<Prefix>) -> Style { Fixed(66).normal() }
+        fn unit(&self, _prefix: Option<Prefix>) -> Style { Fixed(77).bold() }
         fn no_size(&self)          -> Style { Black.italic() }
 
         fn major(&self) -> Style { Blue.on(Red) }
